"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("@slack/client");
/**
 * Report to Slack the result of Danger
 */
function slack(options) {
    const webhook = new client_1.IncomingWebhook(options.webhookUrl);
    const msg = createMessage(danger.github.pr, results, options);
    webhook.send(msg);
}
exports.default = slack;
function createMessage(pr, resultLists, options) {
    const msg = {
        text: "",
        username: options.username || "DangerJS",
        iconEmoji: options.iconEmoji || ":open_mouth:",
        attachments: [],
    };
    // custom iconUrl if set
    if (options.iconUrl) {
        msg.iconUrl = options.iconUrl;
    }
    // custom channel if set
    if (options.channel) {
        msg.channel = options.channel;
    }
    // send only a custom text
    if (options.text) {
        msg.text = `${options.text}`;
    }
    else {
        // send only the report
        const fails = resultLists.fails;
        const warnings = resultLists.warnings;
        const messages = resultLists.messages;
        const markdowns = resultLists.markdowns;
        // temporary wild cast to retrieve url
        // TODO: replace by proper html url property from danger type
        const prInfo = `<${pr.html_url}|*PR#${pr.number}* - ${pr.title}>`;
        const prAuthor = `<${pr.user.html_url}|${pr.user.login}>`;
        msg.iconEmoji = getDynamicEmoji(fails, warnings);
        msg.text = `${prInfo} by ${prAuthor}\n${pr.body}`;
        // add violations as slack attachments
        if (!fails.length && !warnings.length && !messages.length) {
            msg.text += "\nNo output to show.";
        }
        else {
            if (fails.length > 0) {
                msg.attachments.push(createAttachment("Fails", "danger", fails));
            }
            if (warnings.length > 0) {
                msg.attachments.push(createAttachment("Warnings", "warning", warnings));
            }
            if (messages.length > 0) {
                msg.attachments.push(createAttachment("Messages", "#999", messages));
            }
            if (markdowns.length > 0) {
                msg.attachments.push(createMarkdownAttachment("Comments", "#EEE", markdowns));
            }
        }
    }
    return msg;
}
exports.createMessage = createMessage;
function getDynamicEmoji(errors, warnings) {
    const emojiError = ":rage:";
    const emojiWarning = ":neutral_face:";
    const emojiHealthy = ":blush:";
    if (errors.length > 0) {
        return emojiError;
    }
    else if (warnings.length > 0) {
        return emojiWarning;
    }
    else {
        return emojiHealthy;
    }
}
exports.getDynamicEmoji = getDynamicEmoji;
function createAttachment(title, color, violations) {
    const titleWithCount = `${title} (${violations.length})`;
    const textContent = violations.map(violation => `• ${violation.message}`).join("\n");
    return {
        color,
        fallback: titleWithCount,
        title: titleWithCount,
        text: textContent,
        mrkdwn_in: ["text"],
    };
}
exports.createAttachment = createAttachment;
function createMarkdownAttachment(title, color, comments) {
    return {
        color,
        fallback: title,
        title,
        text: comments.join("\n"),
        mrkdwn_in: ["text"],
    };
}
exports.createMarkdownAttachment = createMarkdownAttachment;
